# Import libraries
import csv
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
import sys
import os

"""
This file is used for plotting of speeds and determination factors across three replicates within the same light 
setting. The data needed can be retrieved from detection_tracking.py. 

Update "path" to the path you want to save the plots in, and "light_setting" to the light setting you are working with. 
Further, the name of three input .csv-files need to be updated below "Read and process mean data". 
"""

path = "Plot_blue/"
light_setting = "Blue-OD1-P0"
if not os.path.exists(path):
    os.makedirs(path)

maxInt = sys.maxsize
while True:
    try:
        csv.field_size_limit(maxInt)
        break
    except OverflowError:
        maxInt = int(maxInt/10)

""" Read and process mean data """
data_rep1 = []
with open("data_blue_r1.csv") as a:
    rd_a = csv.DictReader(a, delimiter=',')
    for row in rd_a:
        data_rep1.append(row)

data_rep2 = []
with open("data_blue_r2.csv") as b:
    rd_b = csv.DictReader(b, delimiter=',')
    for row in rd_b:
        data_rep2.append(row)

data_rep3 = []
with open("data_blue_r3.csv") as c:
    rd_c = csv.DictReader(c, delimiter=',')
    for row in rd_c:
        data_rep3.append(row)

data_tot = np.concatenate((np.array(data_rep1), np.array(data_rep2), np.array(data_rep3)), axis=0)

max_speed_1 = []
avg_speed_1 = []
detfac_tot_1 = []
detfac_light_1 = []
detfac_dark_1 = []
speed_tot_1 = []
speed_x_1 = []
start_track_1 = []

max_speed_2 = []
avg_speed_2 = []
detfac_tot_2 = []
detfac_light_2 = []
detfac_dark_2 = []
speed_tot_2 = []
speed_x_2 = []
start_track_2 = []

max_speed_3 = []
avg_speed_3 = []
detfac_tot_3 = []
detfac_light_3 = []
detfac_dark_3 = []
speed_tot_3 = []
speed_x_3 = []
start_track_3 = []

for row in data_rep1:
    max_speed_1.append(float(row['max_speed']))
    avg_speed_1.append(float(row['avg_speed']))
    detfac_tot_1.append(float(row['det_factor_total']))
    detfac_light_1.append(float(row['det_factor_light']))
    detfac_dark_1.append(float(row['det_factor_dark']))
    speed_tot = [float(x) for x in row['speed_tot'].replace('[', '').replace(']', '').split(',')]
    speed_tot_1.append(speed_tot)
    speed_x = [float(a) for a in row['speed_x'].replace('[', '').replace(']', '').split(',')]
    speed_x_1.append(speed_x)
    start_track_1.append(float(row['start_time']))

for row in data_rep2:
    max_speed_2.append(float(row['max_speed']))
    avg_speed_2.append(float(row['avg_speed']))
    detfac_tot_2.append(float(row['det_factor_total']))
    detfac_light_2.append(float(row['det_factor_light']))
    detfac_dark_2.append(float(row['det_factor_dark']))
    speed_tot = [float(y) for y in row['speed_tot'].replace('[', '').replace(']', '').split(',')]
    speed_tot_2.append(speed_tot)
    speed_x = [float(b) for b in row['speed_x'].replace('[', '').replace(']', '').split(',')]
    speed_x_2.append(speed_x)
    start_track_2.append(float(row['start_time']))

for row in data_rep3:
    max_speed_3.append(float(row['max_speed']))
    avg_speed_3.append(float(row['avg_speed']))
    detfac_tot_3.append(float(row['det_factor_total']))
    detfac_light_3.append(float(row['det_factor_light']))
    detfac_dark_3.append(float(row['det_factor_dark']))
    speed_tot = [float(z) for z in row['speed_tot'].replace('[', '').replace(']', '').split(',')]
    speed_tot_3.append(speed_tot)
    speed_x = [float(c) for c in row['speed_x'].replace('[', '').replace(']', '').split(',')]
    speed_x_3.append(speed_x)
    start_track_3.append(float(row['start_time']))

""" Clean speed data """
speed_tot_b1_max = []
speed_tot_a1_max = []
speed_tot_b2_max = []
speed_tot_a2_max = []
speed_tot_b3_max = []
speed_tot_a3_max = []

speed_x_b1_max = []
speed_x_a1_max = []
speed_x_b2_max = []
speed_x_a2_max = []
speed_x_b3_max = []
speed_x_a3_max = []

speed_tot_b1_mean = []
speed_tot_a1_mean = []
speed_tot_b2_mean = []
speed_tot_a2_mean = []
speed_tot_b3_mean = []
speed_tot_a3_mean = []

speed_x_b1_mean = []
speed_x_a1_mean = []
speed_x_b2_mean = []
speed_x_a2_mean = []
speed_x_b3_mean = []
speed_x_a3_mean = []

for i, track in enumerate(start_track_1):
    time_start = start_track_1[i]*128.0
    speed_tot_a = []
    speed_x_a = []
    speed_tot_b = []
    speed_x_b = []
    if time_start < 30*128:
        num_dark = 30*128-time_start
        num_light = len(speed_tot_1[i])-num_dark
    else:
        num_dark = 0.0
        num_light = len(speed_tot_1[i])
    for j in range(len(speed_tot_1[i])):
        if j <= num_dark:
            speed_tot_b.append(speed_tot_1[i][j])
            speed_x_b.append(speed_x_1[i][j])
        else:
            speed_tot_a.append(speed_tot_1[i][j])
            speed_x_a.append(speed_x_1[i][j])

    speed_tot_b1_max.append(np.max(speed_tot_b))
    speed_tot_a1_max.append(np.max(speed_tot_a))
    speed_x_b1_max.append(max(speed_x_b, key=abs))
    speed_x_a1_max.append(max(speed_x_a, key=abs))
    speed_tot_b1_mean.append(np.mean(speed_tot_b))
    speed_tot_a1_mean.append(np.mean(speed_tot_a))
    speed_x_b1_mean.append(np.mean(speed_x_b))
    speed_x_a1_mean.append(np.mean(speed_x_a))

for i, track in enumerate(start_track_2):
    time_start = start_track_2[i]*128.0
    speed_tot_a = []
    speed_x_a = []
    speed_tot_b = []
    speed_x_b = []
    if time_start < 30*128:
        num_dark = 30*128-time_start
        num_light = len(speed_tot_2[i])-num_dark
    else:
        num_dark = 0.0
        num_light = len(speed_tot_2[i])
    for j in range(len(speed_tot_2[i])):
        if j <= num_dark:
            speed_tot_b.append(speed_tot_2[i][j])
            speed_x_b.append(speed_x_2[i][j])
        else:
            speed_tot_a.append(speed_tot_2[i][j])
            speed_x_a.append(speed_x_2[i][j])

    speed_tot_b2_max.append(np.max(speed_tot_b))
    speed_tot_a2_max.append(np.max(speed_tot_a))
    speed_x_b2_max.append(max(speed_x_b, key=abs))
    speed_x_a2_max.append(max(speed_x_a, key=abs))
    speed_tot_b2_mean.append(np.mean(speed_tot_b))
    speed_tot_a2_mean.append(np.mean(speed_tot_a))
    speed_x_b2_mean.append(np.mean(speed_x_b))
    speed_x_a2_mean.append(np.mean(speed_x_a))

for i, track in enumerate(start_track_3):
    time_start = start_track_3[i]*128.0
    speed_tot_a = []
    speed_x_a = []
    speed_tot_b = []
    speed_x_b = []
    if time_start < 30*128:
        num_dark = 30*128-time_start
        num_light = len(speed_tot_3[i])-num_dark
    else:
        num_dark = 0.0
        num_light = len(speed_tot_3[i])
    for j in range(len(speed_tot_3[i])):
        if j <= num_dark:
            speed_tot_b.append(speed_tot_3[i][j])
            speed_x_b.append(speed_x_3[i][j])
        else:
            speed_tot_a.append(speed_tot_3[i][j])
            speed_x_a.append(speed_x_3[i][j])

    speed_tot_b3_max.append(np.max(speed_tot_b))
    speed_tot_a3_max.append(np.max(speed_tot_a))
    speed_x_b3_max.append(max(speed_x_b, key=abs))
    speed_x_a3_max.append(max(speed_x_a, key=abs))
    speed_tot_b3_mean.append(np.mean(speed_tot_b))
    speed_tot_a3_mean.append(np.mean(speed_tot_a))
    speed_x_b3_mean.append(np.mean(speed_x_b))
    speed_x_a3_mean.append(np.mean(speed_x_a))


speed_tot_max = [speed_tot_b1_max, speed_tot_a1_max, speed_tot_b2_max, speed_tot_a2_max,
                 speed_tot_b3_max, speed_tot_a3_max]
speed_x_max = [speed_x_b1_max, speed_x_a1_max, speed_x_b2_max, speed_x_a2_max, speed_x_b3_max, speed_x_a3_max]
speed_tot_mean = [speed_tot_b1_mean, speed_tot_a1_mean, speed_tot_b2_mean, speed_tot_a2_mean,
                  speed_tot_b3_mean, speed_tot_a3_mean]
speed_x_mean = [speed_x_b1_mean, speed_x_a1_mean, speed_x_b2_mean, speed_x_a2_mean, speed_x_b3_mean, speed_x_a3_mean]

max_speed = 0
rep = 0
for i, element in enumerate(speed_tot_max):
    if np.max(element) > max_speed:
        max_speed = np.max(element)
        rep = i
print('max speed', max_speed)
print('rep', rep)

max_speed_x = 0
rep_x = 0
for j, elementx in enumerate(speed_x_max):
    if max(elementx, key=abs) < max_speed_x:
        max_speed_x = max(elementx, key=abs)
        rep_x = j
print('max speed x', max_speed_x)
print('rep x', rep_x)

colors = [sns.light_palette("green", n_colors=6), sns.light_palette("yellow", n_colors=6),
          sns.light_palette("red", n_colors=6)]
new_colors = []
for element in colors:
    new_colors.append(list(element[1]))
    new_colors.append(list(element[0]))


max_speed = [[x for x in max_speed_1 if str(x) != 'nan'], [y for y in max_speed_2 if str(y) != 'nan'],
             [z for z in max_speed_3 if str(z) != 'nan']]
avg_speed = [[a for a in avg_speed_1 if str(a) != 'nan'], [b for b in avg_speed_2 if str(b) != 'nan'],
             [c for c in avg_speed_3 if str(c) != 'nan']]
detfac_tot_1 = [d for d in detfac_tot_1 if str(d) != 'nan']
detfac_tot_2 = [e for e in detfac_tot_2 if str(e) != 'nan']
detfac_tot_3 = [f for f in detfac_tot_3 if str(f) != 'nan']
detfac_tot = [detfac_tot_1, detfac_tot_2, detfac_tot_3]

detfac_dark_1 = [g for g in detfac_dark_1 if str(g) != 'nan']
detfac_dark_2 = [h for h in detfac_dark_2 if str(h) != 'nan']
detfac_dark_3 = [i for i in detfac_dark_3 if str(i) != 'nan']
detfac_dark = [detfac_dark_1, detfac_dark_2, detfac_dark_3]

detfac_light_1 = [j for j in detfac_light_1 if str(j) != 'nan']
detfac_light_2 = [k for k in detfac_light_2 if str(k) != 'nan']
detfac_light_3 = [l for l in detfac_light_3 if str(l) != 'nan']
detfac_light = [detfac_light_1, detfac_light_2, detfac_light_3]

placing_txt = [1.5, 3.5, 5.5]
det_factors = [detfac_dark_1, detfac_light_1, detfac_dark_2, detfac_light_2, detfac_dark_3, detfac_light_3]

""" Figure 1: box plot max speeds"""
plt.figure(1)
plt.boxplot(max_speed, positions=[1, 3, 5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1, 3, 5], ["R1", "R2", "R3"])
plt.ylabel("Speed [mm/s]")
plt.title("Max speed, {}".format(light_setting))
plt.savefig(path + "box_max_speed.png")

""" Figure 2: box plot average speeds"""
plt.figure(2)
plt.boxplot(avg_speed, positions=[1, 3, 5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1, 3, 5], ["R1", "R2", "R3"])
plt.ylabel("Speed [mm/s]")
plt.title("Average speed, {}".format(light_setting))
plt.savefig(path + "box_avg_speed.png")


""" Figure 3: box plot determination factors """
plt.figure(3)
box = plt.boxplot(det_factors, patch_artist=True,
                  positions=[1, 1.5, 3, 3.5, 5, 5.5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1.25, 3.25, 5.25], ["R1", "R2", "R3"], fontsize='x-large')
means = [np.median(x) for x in det_factors]
diff_list = []
for i in range(0, int(len(means)) - 1, 2):
    diff_list.append((means[i+1] - means[i]))
for i, patch in enumerate(box['boxes']):
    patch.set_facecolor(new_colors[i])
positions=[1, 1.5, 3, 3.5, 5, 5.5]
for n, line in enumerate(box['medians']):
    x, y = line.get_xydata()[1]
    plt.text(x - 0.24, y + 0.01, '%.2f' % y, size='medium', horizontalalignment='center', verticalalignment='center')
plt.title("Determination factors before and after light is turned on, {}".format(light_setting))
plt.savefig(path + "box_det_factors.png")


""" Figure 4: box plot max total speed """
plt.figure(4)
box = plt.boxplot(speed_tot_max, patch_artist=True,
                  positions=[1, 1.5, 3, 3.5, 5, 5.5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1.25, 3.25, 5.25], ["R1", "R2", "R3"], fontsize='x-large')
means = [np.median(x) for x in speed_tot]
diff_list = []
for i in range(0, int(len(means)) - 1, 2):
    diff_list.append((means[i+1] - means[i]))
for i, patch in enumerate(box['boxes']):
    patch.set_facecolor(new_colors[i])
positions=[1, 1.5, 3, 3.5, 5, 5.5]
for n, line in enumerate(box['medians']):
    x, y = line.get_xydata()[1]
    plt.text(x - 0.24, y + 1.29, '%.2f' % y, size='large', horizontalalignment='center', verticalalignment='center')
plt.ylabel("Speed [mm/s]", fontsize='x-large')
plt.title("Max speed before and after light is turned on, {}".format(light_setting))
plt.savefig(path + "box_tot_max_speed.png")


""" Figure 5: box plot max speed in x-direction"""
plt.figure(5)
box = plt.boxplot(speed_x_max, patch_artist=True,
                  positions=[1, 1.5, 3, 3.5, 5, 5.5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1.25, 3.25, 5.25], ["R1", "R2", "R3"], fontsize='x-large')
means = [np.median(x) for x in speed_tot]
diff_list = []
for i in range(0, int(len(means)) - 1, 2):
    diff_list.append((means[i+1] - means[i]))
for i, patch in enumerate(box['boxes']):
    patch.set_facecolor(new_colors[i])
positions=[1, 1.5, 3, 3.5, 5, 5.5]
for n, line in enumerate(box['medians']):
    x, y = line.get_xydata()[1]
    plt.text(x - 0.24, y + 1.27, '%.2f' % y, size='medium', horizontalalignment='center', verticalalignment='center')
plt.ylabel("Speed [mm/s]", fontsize='x-large')
plt.title("Max speed in x-direction before and after light is turned on, {}".format(light_setting))
plt.savefig(path + "box_x_max_speed.png")

""" Figure 6: box plot mean total speed """
plt.figure(6)
box = plt.boxplot(speed_tot_mean, patch_artist=True,
                  positions=[1, 1.5, 3, 3.5, 5, 5.5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1.25, 3.25, 5.25], ["R1", "R2", "R3"])
means = [np.median(x) for x in speed_tot]
diff_list = []
for i in range(0, int(len(means)) - 1, 2):
    diff_list.append((means[i+1] - means[i]))
for i, patch in enumerate(box['boxes']):
    patch.set_facecolor(new_colors[i])
positions = [1, 1.5, 3, 3.5, 5, 5.5]
for n, line in enumerate(box['medians']):
    x, y = line.get_xydata()[1]
    plt.text(x - 0.22, y + 0.01, '%.2f' % y, size='small', horizontalalignment='center', verticalalignment='center')
plt.ylabel("Speed [mm/s]")
plt.title("Mean speed before and after light is turned on, {}".format(light_setting))
plt.savefig(path + "box_tot_mean_speed.png")


""" Figure 7: box plot mean speed in x-direction"""
plt.figure(7)
box = plt.boxplot(speed_x_mean, patch_artist=True,
                  positions=[1, 1.5, 3, 3.5, 5, 5.5], widths=0.5, showmeans=True, meanline=True)
plt.xticks([1.25, 3.25, 5.25], ["R1", "R2", "R3"])
means = [np.median(x) for x in speed_tot]
diff_list = []
for i in range(0, int(len(means)) - 1, 2):
    diff_list.append((means[i+1] - means[i]))
for i, patch in enumerate(box['boxes']):
    patch.set_facecolor(new_colors[i])
positions=[1, 1.5, 3, 3.5, 5, 5.5]
for n, line in enumerate(box['medians']):
    x, y = line.get_xydata()[1]
    plt.text(x - 0.22, y + 0.01, '%.2f' % y, size='small', horizontalalignment='center', verticalalignment='center')
plt.ylabel("Speed [mm/s]")
plt.title("Mean speed in x-direction before and after light is turned on, {}".format(light_setting))
plt.savefig(path + "box_x_mean_speed.png")

